use ratatui::crossterm::{event::{self, Event, KeyCode, KeyEvent, KeyEventKind}};
use ratatui::{
    buffer::Buffer,
    layout::{Constraint, Flex, Layout, Rect},
    style::Stylize,
    symbols::border,
    text::{Line, Text},
    widgets::{Block, Borders, Clear, Paragraph, Widget, Wrap},
    DefaultTerminal,
    Frame,
};

use crate::{message::{Message, MessageKind}, player::Player, ui::ui};

#[derive(Debug, Default)]
pub struct Client {
    pub addr: String,
    pub client: reqwest::Client,
    pub user: Option<Player>,
    pub name: String,
    pub exit: bool,
    pub popup: bool,
    pub response: String,
}

impl Client {
    pub async fn run(&mut self, terminal: &mut DefaultTerminal) -> anyhow::Result<()> {
        self.addr = "http://120.0.0.1:8080".to_string();
        self.client = reqwest::Client::new();

        while !self.exit {
            terminal.draw(|frame| ui(frame, self))?;
            crate::ui::handle_events(self).await?;
        }

        Ok(())
    }

    pub fn exit(&mut self) {
        self.exit = true;
    }

    pub async fn send_player(&mut self, player: &Player, addr: &str) -> anyhow::Result<()> {
        let player_message = Message::new(
            &player.get_name(),
            MessageKind::CreatePlayer,
            player.encode()?
        )?;

        let response = self.client
            .post([&addr, "/create/player"].concat())
            .header("Content-Type", "application/json")
            .body(player_message.encode()?)
            .send().await;

        if let Err(e) = &response {
            self.response = e.to_string();
            return Ok(())
        } else {
            self.response = format!("{:?}", response?);
        }

        Ok(())
    }
}

impl Widget for &Client {
    fn render(self, area: Rect, buf: &mut Buffer) {
        let title = Line::from(" durak tui test fre ".bold());

        let instructions = Line::from(vec![
            " create player ".into(),
            "1".blue().bold(),
            " create lobby ".into(),
            "2".blue().bold(),
            " quit ".into(),
            "<Q>".blue().bold(),
        ]);

        let block = Block::bordered()
            .title(title.centered())
            .title_bottom(instructions.centered())
            .border_set(border::THICK);

        let infos = Text::from(vec![
            Line::from(vec![
                "connected to: ".into(),
                self.addr.clone().yellow(),
            ]),
            Line::from(vec![
                "username: ".into(),
                self.name.clone().yellow(),
            ]),
        ]);

        Paragraph::new(infos)
            .left_aligned()
            .block(block)
            .render(area, buf);
    }
}
